/*
* This file is part of the XKCD Velociraptor Simulation originally written by Matthew Beckler, Grant Miller, and John Saxton.
* 
* the XKCD Velociraptor Simulation is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* The XKCD Velociraptor Simulation is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with the XKCD Velociraptor Simulation; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**/

import java.util.*;
import java.io.*;

public class RaptorEvaluate
{
	private final static double STEP = 0.01; //this is the timestep

	public static ArrayList evaluate(Victim[] runners) //runners will have length of 80
	{
		Raptor injuredRaptor = new Raptor(10, 0, 11.53); //injured, so it can only run at 10m/s
		Raptor leftRaptor = new Raptor(25, -10, -5.77);
		Raptor rightRaptor = new Raptor(25, 10, -5.77);
		ArrayList results = new ArrayList(runners.length); //there is a custom data-class called Result, which will go into results
		
		for (int i = 0; i < runners.length; i++) //test each runner, in order
		{
			boolean runnerAlive = true;
			double time = 0.0;
			Victim joe = runners[i]; //I really hate joe
			
			//System.out.println("Testing Runner #" + i + " " + joe);
			
			try
			{
				//yes, it's true, it writes and overwrites output.csv for each and every runner, and we only get to see the last one
				//a teriffic waste of computer resources, but oh well...
				BufferedWriter out = new BufferedWriter(new FileWriter("output.csv"));
				
				while (runnerAlive)
				{	
					out.write(time + ", " + joe.getX() + ", " + joe.getY() + ", " + 
							injuredRaptor.getX() + ", " + injuredRaptor.getY() + ", " +
							leftRaptor.getX() + ", " + leftRaptor.getY() + ", " +
							rightRaptor.getX() + ", " + rightRaptor.getY() + "\n");

					runnerAlive = !injuredRaptor.updatePosition(STEP, joe) && 
						!leftRaptor.updatePosition(STEP, joe) && 
						!rightRaptor.updatePosition(STEP, joe); //if none of the raptors caught Joe, then he's still alive
					joe.updatePosition(STEP, joe); //run joe run!

					time += STEP;
				}
				out.close();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			
			//System.out.println("  Survived for " + time + " seconds");
			results.add(new Result(joe, time)); //we'll use the reference to joe to get his offspring, if he was one of the best ones
			
			//move these guys back to the start
			injuredRaptor.reset();
			leftRaptor.reset();
			rightRaptor.reset();
		}
		
		//we only want to take the best runners, so we sort the results, using a custom sorting method (see below)
		Collections.sort(results);
		return results;
	}
	
	public static void main(String arrrrgs[])
	{
		Victim[] runners = new Victim[80];
		//so basically we start with 8 runners, headed E, NE, N, NW, W, SW, S, SE
		//then we mutate them 10 times, to get our 80 initial runners
		
		//direction is how many eighths of a circle we are currently looking at
		for (int direction = 0; direction < 8; direction++)
		{
			Victim adam = new Victim(6, 0, 0, direction * 0.785398, 0.2); //runs 6 m/s, start at origin, 0.2 mutation factor, variable theta (eighths of a circle)
			//Note about the 0.2 mutation factor, this equates to about a 12 degree angle
			for (int i = 0; i < 10; i++)
			{
				runners[direction * 10 + i] = adam.mitosis();
			}
		}
		
		//we keep track of the best time and a reference to the best runner
		double bestTime = 0;
		Victim bestRunner = null;
		
		int generationsWithoutImprovement = 0;
		
		//how many generations to go without seeing any improvement, 5 usually works, we went with 15 just to be sure
		int maxGenerationsWithoutImprovement = 15;

		int currentGeneration = 0;

		System.out.println("Starting test with 80 runners");

		while (generationsWithoutImprovement < maxGenerationsWithoutImprovement)
		{
			ArrayList results = evaluate(runners); //this returns a sorted ArrayList of the results, sorted by best times at the top
			
			double currentGenerationBestTime = ((Result)results.get(0)).time;
			Victim currentGenerationBestRunner = ((Result)results.get(0)).v;
			
			System.out.println("Generation #" + currentGeneration + " best runner = " + currentGenerationBestRunner + " with time = " + currentGenerationBestTime);
			
			if (currentGenerationBestTime > bestTime)
			{
				generationsWithoutImprovement = 0;
				bestTime = currentGenerationBestTime;
				bestRunner = currentGenerationBestRunner;
			}
			else
			{
				generationsWithoutImprovement++;
			}
			
			//here, we take the 8 best runners, mutate them 10 times each, and re-run the simulation
			for (int i = 0; i < 8; i++)
			{
				for (int c = 0; c < 10; c++)
				{
					runners[i * 10 + c] = ((Result) results.get(i)).v.mitosis();
				}
			}
			currentGeneration++;
		}
		System.out.println("Best runner at the end: " + bestRunner + " with time = " + bestTime);
	}
}

class Result implements Comparable
{
	public Victim v;
	public double time;
	
	public Result(Victim v, double time)
	{
		this.v = v;
		this.time = time;
	}
	
	public String toString()
	{
		return "(" + v.getTheta() + ", " + time + ")";
	}
	
	public int compareTo(Object o)
	{
		Result r = (Result) o;
		if (r.time < time)
		{
			return -1;
		}
		else if (r.time > time)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
}
