/*
* This file is part of the XKCD Velociraptor Simulation originally written by Matthew Beckler, Grant Miller, and John Saxton.
* 
* the XKCD Velociraptor Simulation is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* The XKCD Velociraptor Simulation is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with the XKCD Velociraptor Simulation; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**/

import java.util.Random;

public class Victim extends Movable
{
	//after consulting the XKCD author, we decided to allow the victim to only sprint radially outward at a constant angle, called theta
	private double theta;

	//as part of our genetic algorithm to evolve to the best victim possible, based on survival time, we introduced a mutationRate factor, which determines how much mutation happens to this Victim's offspring. See more info in the mitosis method below
	private double mutationRate;

	//since theta is a constant, we only need to calculate dx and dy at construction
	private double dx;
	private double dy;
	private static Random r = new Random();
	
	public Victim(double v, double x, double y, double theta, double mutationRate)
	{
		super(v, x, y);
		this.theta = theta;
		this.mutationRate = mutationRate;
		dx = Math.cos(theta) * v;
		dy = Math.sin(theta) * v;
	}
	
	public double getTheta()
	{
		return theta;
	}
	
	public boolean updatePosition(double dt, Movable target)
	{
		x += dx * dt;
		y += dy * dt;
		
		return false;
	}
	
	public Victim mitosis()
	{
		//nextGaussian "Returns the next pseudorandom, Gaussian ("normally") distributed double value with mean 0.0 and standard deviation 1.0 from this random number generator's sequence."
		double mutationAmount = r.nextGaussian();
		//after getting the mutationAmount, we scale it by the mutationRate, and add it to theta

		//this variable is the scaling factor for the mutationRate. It either doubles or halves each time. It's kind of interesting, how the mutationRate declines as the simulation progresses, as the Victims evolve to be very very close to perfect, it's not advantageous to have wildly varying offspring.
		double mutationRateMutationAmount = (r.nextBoolean() ? 2.0 : 0.5);
		return new Victim(v, 0, 0, theta + (mutationAmount * mutationRate), mutationRate * mutationRateMutationAmount);
	}
	
	public String toString()
	{
		return "(" + theta + ", " + mutationRate + ")";
	}
}
